#include "lib/structure.h"
#include "lib/noiseFbm.h"
#include "lib/colorCorrection.h"

vec3 skyc(function m, Color n)
{
    vec3 scol = mix(vec3(0.0,0.43,0.9)*1.5, vec3(0.2,0.4,0.8)*0.8, m.sunfunc);
    scol = mix(scol, vec3(0.0, 0.05, 0.12)*1.3, m.nightfunc);
    scol = jodie_reinhard_tonemap(scol);

    vec3 col = mix(scol, vec3(n.fc.r, n.fc.g, n.fc.b*1.2)*0.6, clamp(m.lp,0.,1.));

    col = mix(col, 
vec3(n.fc.r+n.fc.b*0.1,n.fc.g*0.5+n.fc.r*0.6, n.fc.r*0.5+n.fc.b*0.5)+0.1*(n.fc.r*n.fc.g*n.fc.b), clamp(pow(m.lp*1.3, 1.0), 0.,1.));
    col = mix(col, n.fc.rgb, clamp(pow(m.lp, 4.),0.,1.));

    col = mix(col, vec3(scol)*1.6, clamp(m.lp*1.3,0.,1.)*m.nightfunc);
    col = mix(col, vec3(scol),
clamp(pow(m.lp,4.)*1.8,0.,1.)*m.nightfunc);

     col = mix(col, vec3(n.fc.rgb*1.3), m.rainfunc);
     col = mix(col, vec3(n.fc.rgb*1.5),
clamp(m.lp*1.5,0.,1.)*m.rainfunc);
     col = mix(col, vec3(n.fc.rgb*1.5),
clamp(pow(m.lp,4.)*1.5,0.,1.)*m.rainfunc);

if((n.fc.b > 0.15 && (n.fc.b > n.fc.g*1.2) && (n.fc.b >= n.fc.r*1.8) && (n.fc.g > n.fc.r*1.6)) || (n.fc.g*1.2 >= n.fc.b) && (n.fc.r*1.3 < n.fc.g)){
    col = mix(col, vec3(n.fc.rgb), m.rainfunc);
}
return col;
}

float draw_cloud(highp vec3 pos, function m)
{
    highp float move = TIME;
    vec3 coord = pos*vec3(0.5,0.5,0.5)*0.35+move*vec3(-0.1,0.,0.1)*0.2;
    float s = 0.0;
    float tot = 0.0;

    for(float i = 0.0; i < 4.0; i++){
    s += noise(coord*exp(i*1.03)*0.6+move*i*vec3(0.1,0.,0.1)*0.2)*exp(-i*0.8);
    tot += exp(-i*0.8);
	}

return 1.0-pow(0.4,max(s/tot-0.51+m.rainfunc*0.1,0.0)*2.2);
}

vec4 render_clouds(highp vec3 pos, highp vec3 mpos, float py, function m)
{

    float steps = 13.0;
    float thick = mix(32.0, 3.0, m.rainfunc) / steps;
    float sc = 40.0;
    float o = clamp(py, 0.0, 0.4) * 2.5 + 0.7, t = 1.0, a = 0.0;

    vec3 sum = vec3(0);
    vec4 result = vec4(0);

    float dither = dithering(pos*100.0)*0.1;

    for(float i = 0.0; i < steps; i++){

    float h = i / sc;
    float denc = draw_cloud(pos + dither, m);
    float m = exp(-mix(0.8, 0.4, m.rainfunc) * denc * thick); 

    t *= m;

    sum += t * (exp(h) / 1.3) * denc * thick;
    a += (1.0 - m) * (1.0 - a);

    pos += mpos;
    }

    result = vec4(sum, a);
    result.a = mix(result.a, 0.0, min(pow(o, 10.0), 1.0));
    result = vec4(result.rgb, clamp((result.a-0.6)*1.6, 0.0, 1.0)) * result.a;

return result;
}
